@@ -5,60 +5,41 @@
 #       Copyright (c) 2011 R Pratap Chakravarthy
 
 import os, fcntl, signal, sys, time, threading, imp, signal
+from   os.path  import abspath
 
 from   pluggdapps.plugin        import implements, ISettings, Singleton, \
                                        pluginname
 from   pluggdapps.interfaces    import ICommand, IHTTPServer
 import pluggdapps.utils         as h
 
-_default_settings = h.ConfigDict()
-_default_settings.__doc__ = (
-    "Configuration for serve sub-command."
-)
-
-_default_settings['IHTTPServer'] = {
-    'default' : 'httpepollserver',
-    'types'   : (str,),
-    'help'    : "Plugin name implementing :class:`IHTTPServer`."
-}
-_default_settings['reload'] = {
-    'default' : False,
-    'types'   : (bool,),
-    'help'    : "Whether to monitor for changes to module files and watched "
-                "files, and restart the server."
-}
-_default_settings['reload.config'] = {
-    'default' : True,
-    'types'   : (bool,),
-    'help'    : "This parameter specifies whether the server should be "
-                "restarted when a configuration parameter is changed."
-}
-_default_settings['reload.poll_interval'] = {
-    'default' : 1,
-    'types'   : (int,),
-    'help'    : "Number seconds to poll for watched file's modification "
-                "timestamp. When a file is modified server is restarted."
-}
-_default_settings['reload.config'] = {
-    'default' : True,
-    'types'   : (bool,),
-    'help'    : "This parameter specifies whether the server should be "
-                "restarted when a configuration parameter is changed."
-}
-_default_settings['reload.poll_interval'] = {
-    'default' : 1,
-    'types'   : (int,),
-    'help'    : "Number seconds to poll for watched file's modification "
-                "timestamp. When a file is modified server is restarted."
-}
-
 class CommandServe( Singleton ):
-    """Sub-command to starts a web server (using epoll) for pluggdapps."""
+    """Sub-command for starting native web server. Configuring this plugin
+    does not control the web server, instead refer to the corresponding web
+    server plugin. By default it uses :class:`HTTPEPollServer`, a single
+    threaded / single process epoll based server.
+
+    For automatic server restart, when a module or configuration file is
+    modified, pass ``-m`` switch to main script and ``-r`` switch to this
+    sub-command. Typically used in development mode,
+    
+    .. code-block:: bash
+
+        $ pa -w -m -c <master.ini> serve -r
+
+    .. code-block:: text
+
+        fork ---> child ------> poll-thread
+          |        |      |
+          *--------*      |
+           monitor        *---> pluggdapps-thread
+
+    """
 
     implements( ICommand )
 
     description = "Start epoll based http server."
     cmd = 'serve'
+
     def __init__( self, *args, **kwargs ):
         self.module_mtimes = {}
 
@@ -81,28 +62,30 @@ class CommandServe( Singleton ):
     #---- Local function.
 
     def gemini( self, args ):
-        """If reload is enabled, then create a thread to poll for changing
-        files, based on mtime, and serve http requests. When a file gets
-        changed, return with a predefined exit code so that waiting process
-        will restart the gemini server again.
-
-        Typically used in development mode.
-        """
+        """Start a poll thread and then start pluggdapps platform."""
+        server = self.query_plugin( IHTTPServer, self['IHTTPServer'] )
         if args.mreload :
             # Launch a thread to poll and then start serving http
             t = threading.Thread( target=self.pollthread, 
-                                  name='Reloader', args=(args,) )
+                                  name='Reloader', args=(args, server) )
             t.setDaemon(True)
             t.start()
 
         time.sleep(0.5) # To allow the poll-thread to execute first.
         self.pa.start() # Start pluggdapps
-        server = self.query_plugin( IHTTPServer, self['IHTTPServer'] )
         server.start()  # Blocking call
 
+        # use os._exit() here and not sys.exit() since within a
+        # thread sys.exit() just closes the given thread and
+        # won't kill the process; note os._exit does not call
+        # any atexit callbacks, nor does it do finally blocks,
+        # flush open files, etc.  In otherwords, it is rude.
+        os._exit(3)
 
     def fork_and_monitor( self, args ):
-        """Install the reloading monitor."""
+        """Fork a child process with same command line arguments except the
+        ``-m`` switch. Monitor and reload the child process until normal
+        exit."""
         while True :
             self.pa.logdebug( "Forking monitor ..." )
             pid = os.fork()
@@ -112,9 +95,7 @@ class CommandServe( Singleton ):
                 cmdargs.append( os.environ )
                 h.reseed_random()
                 os.execlpe( sys.argv[0], *cmdargs )
-                # signal.signal( signal.SIGIO, self._watch_handler )
-                # self._watch_files( args )
-                # self.gemini( args )
+
             else :          # parent 
                 try :
                     pid, status = os.wait()
@@ -124,30 +105,28 @@ class CommandServe( Singleton ):
                     sys.exit(0)
                     
 
-    def pollthread( self, args ):
+    def pollthread( self, args, server ):
         """Thread (daemon) to monitor for changing files."""
         self.pa.logdebug( "Periodic poll started for module reloader ..." )
         while True:
             if self.pollthread_checkfiles( args ) == True :
-                # use os._exit() here and not sys.exit() since within a
-                # thread sys.exit() just closes the given thread and
-                # won't kill the process; note os._exit does not call
-                # any atexit callbacks, nor does it do finally blocks,
-                # flush open files, etc.  In otherwords, it is rude.
-                os._exit(3)
+                server.stop()
                 break
             time.sleep( self['reload.poll_interval'] )
 
     def pollthread_checkfiles( self, args ):
         """Check whether any of the module files have modified after loading
         this platform. If so, return True else False."""
-        files = [args.config] if args.config else []
+        from pluggdapps import papackages
         modfiles = {}
         for mod in sys.modules.values() :
             if hasattr( mod, '__file__' ) :
                 modfiles.setdefault( getattr(mod, '__file__'), mod )
+        
+        inifiles = self.inifiles() if self['reload.config'] else []
+        files = list(modfiles.keys()) + self.ttlfiles() + inifiles
 
-        for filename in list( modfiles.keys() ) + files :
+        for filename in files :
             stat = os.stat(filename)
             mtime = stat.st_mtime if stat else 0
 
@@ -164,6 +143,25 @@ class CommandServe( Singleton ):
                 return True
         return False
 
+    def inifiles( self ):
+        """Return a list of .ini files that are related to this
+        environment."""
+        if self.pa.inifile :
+            inifiles = [ abspath(self.pa.inifile) ]
+            inifiles.extend( map( lambda x : x[2], self.pa.webapps.keys() ))
+        else :
+            inifiles = []
+        return inifiles
+
+    def ttlfiles( self ):
+        """Return a list of ttlfile related to this environment."""
+        from pluggdapps import papackages
+        ttlfiles = h.flatten(
+            [ list( map( h.abspath_from_asset_spec, n['ttlplugins'] ))
+              for nm, n in papackages.items() if n['ttlplugins'] ]
+        )
+        return ttlfiles + self.pa._monitoredfiles
+
     #---- ISettings interface methods
 
     @classmethod
@@ -176,9 +174,8 @@ class CommandServe( Singleton ):
     def normalize_settings( cls, sett ):
         """:meth:`pluggdapps.plugin.ISettings.normalize_settings` interface 
         method."""
-        sett['reload'] = h.asbool( sett['reload'] )
         sett['reload.config'] = h.asbool( sett['reload.config'] )
-        sett['reload.poll_interval'] = h.asbool( sett['reload.poll_interval'] )
+        sett['reload.poll_interval'] = h.asint( sett['reload.poll_interval'] )
         return sett
 
 
@@ -210,3 +207,31 @@ class CommandServe( Singleton ):
 def SIGINT_handler( signal, frame ):
     print( 'You pressed Ctrl+C!' )
     sys.exit(0)
+
+
+_default_settings = h.ConfigDict()
+_default_settings.__doc__ = CommandServe.__doc__
+
+_default_settings['IHTTPServer'] = {
+    'default' : 'HTTPEPollServer',
+    'types'   : (str,),
+    'help'    : "Plugin name implementing :class:`IHTTPServer`. This is the "
+                "actual web server that will be started by the sub-command. "
+                "Can be modified only in the .ini file.",
+    'webconfig' : False,
+}
+_default_settings['reload.config'] = {
+    'default' : True,
+    'types'   : (bool,),
+    'help'    : "Relevant when the sub-command is invoked with monitor and "
+                "reload switch. Specifies whether the server should be "
+                "restarted when a configuration file (.ini) is changed."
+}
+_default_settings['reload.poll_interval'] = {
+    'default' : 1,
+    'types'   : (int,),
+    'help'    : "Relevant when the sub-command is invoked with monitor and "
+                "reload switch. Number of seconds to poll for file "
+                "modifications. When a file is modified, server is restarted."
+}
+
