@@ -4,100 +4,160 @@
 # file 'LICENSE', which is part of this source code package.
 #       Copyright (c) 2011 R Pratap Chakravarthy
 
-"""
-Parse configuration settings from various sources (at present ini-files are
-supported), aggregate them, make them avialable on plugins instances. It also
-provides methods for logging error / warning messages.
+"""This module along with the :mod:`plugin` module implement the component
+architechture. We expect that most of the logic written using pluggdapps will,
+one way or the other, be organised as plugins implementing one or more
+interface(s). This module,
+
+* provide platform base class in whose context plugins are instantiated.
+* responsible for parsing configuration parameters from various sources,
+  aggregate them, make them avialable on plugin instances.
+* supply methods for logging error and warning messages.
+
+The platform is instantiated by calling the :meth:`Pluggdapps.boot` method.
+
+**Pre-boot and boot**
+
+Pluggdapps component architecture is always instantiated in the context of a
+platform defined by :class:`Pluggdapps` class or by classes deriving from
+`Pluggdapps`.
+
+Platform boots in two phase, first there is a pre-boot which more or less
+does every thing that is done during an actual boot and then the actual
+booting. Pre-booting is designed such a way that other pluggdapps-packages can
+take part in platform booting, such as loading dynamic plugins, pre-compiling
+template scripts etc ...
+
+So here is what happens duing pre-booting,
+
+* all pluggdapps packages are loaded. But entry points are not called.
+* component system is initialized by calling 
+  :func:`pluggdapps.plugin.plugin_init`.
+* a plain vanilla platform is instantiated using :class:`Pluggdapps`.
+* configuration settings from .ini files and database backend, if available,
+  is loaded.
+
+Pre-booting comes to an end by a call to :func:`pluggdapps.initialize`
+function, which in turn is responsible for re-loading pluggdapps packages and 
+calling the package entry-point. Note that package re-loading is handle in the
+context of a plain vanilla platform instantiated during pre-boot phase.
+
+During the actual boot phase, everything that was done in pre-booting phase
+is repeated. But the entire blue-print of interfaces and plugins from all the
+installed packages will be remembered in the context of, probably a more
+sophisticated :class:`Webapps`, platform class. The choice of platform class
+depends on how the user started pluggdapps.
 
 Configuration:
 --------------
 
-Platform's boot method expect an ini-file regarded as master configuration
-file. Settings in this master configuration file will override package default
-settings for plugins and other special sections. Any configuration section not
-prefixed with **plugin:** is considered as special section, which are explained
-further below. Master configuration file can refer to other configuration file.
+Like mentioned before, platform classes are responsible for handling
+configuration.
+
+**default configuration,**
+
+All plugins deriving from :class:`pluggdapps.plugin.Plugin` class, which is
+how plugins are authored, will automatically implement 
+ISettings interface. And configurable plugins must override `ISettings`
+interface methods. Refer to :class:`pluggdapps.plugin.ISettings` interface to
+learn more about their interface methods. When platform is booted,
+default_settings from loaded plugins will be gathered and remembered.
+
+**ini file,**
+
+Platform is typically booted by supplying an ini-file regarded as master
+configuration file. Settings in this master configuration file will override
+package default settings for plugins and other `special sections`.
+Configuration sections not prefixed with ``plugin:`` is considered as special
+section which are explained further below.
+
+Master configuration file can refer to other configuration file.
   
-An example master configuration file,::
+An example master configuration file,
 
-  base.ini
-  --------
+.. code-block:: ini
 
-  [DEFAULT]
-  <option> = <value>
-  ...
+    [DEFAULT]
+    <option> = <value>
+    ... = ...
 
-  [pluggdapps]
-  <option> = <value>
-  ...
+    [pluggdapps]
+    <option> = <value>
+    ... = ...
 
-  [plugin:<pluginname>]
-  <option> = <value>
-  ...
+    [plugin:<pluginname>]
+    <option> = <value>
+    ... = ...
 
-  [plugin:<pluginname>]
-  ...
+    [...]
 
 Special sections:
 -----------------
 
-**[DEFAULT]** special section, settings that are applicable to all
-configuration sections including plugins and refered configuration files.
+**[DEFAULT]** special section. Settings from this section is applicable to all
+configuration sections including plugins and referred configuration files.
 Semantic meaning of [DEFAULT] section is same as described by ``configparser``
-module from stdlib.
+module from python stdlib.
 
-**[pluggdapps]** special section, settings that are applicable to pluggdapps
-platform typically handled by :class:`Pluggdapps`.
+**[pluggdapps]** special section. Settings from this section is applicable to
+pluggdapps platform typically handled by :class:`Pluggdapps`.
 
-**[mountloc]** special section, is specific to web-framework (explained below)
+**[mountloc]** special section. Specific to web-framework (explained below)
 that is built on top of pluggdapps component architecture. Provides 
-configuration settings on how to mount web-applications on web-url. Typically
-handled by :class:`Webapps`.
+configuration settings on how to mount web-applications on web-url. Handled
+by :class:`Webapps` platform class.
+
+To learn more about backend store for configuration settings refer to module,
+:mod:`pluggdapps.config`.
   
 Web-application platform:
 -------------------------
 
 Implemented by :class:`Webapps` class (which derives from base platform class
 :class:`Pluggdapps`), it can host any number web-application, and/or instance
-of same web-application, in a single python environment. Every web-application
+of same web-application in single python environment. Every web-application
 is a plugin implementing :class:`pluggdapps.interfaces.IWebApp` interface.
-When plugins are instantiated by a webapp plugin, either directly or
-indirectly, the instantiated plugins are automatically supplied with
-**.webapp** attribute.
+Like mentioned above every plugin gets instantiated in the context of a
+platform, and in this case, when plugins are instantiated by `IWebApp` plugin,
+either directly or indirectly, the instantiated plugins are automatically
+supplied with **.webapp** attribute which is now part of its context.
+
+**[mountloc] section and application wise ini file,**
 
 Web-applications can be mounted, hosted, on a netlocation and script-path
 (collectively called as ``netpath``). This is configured under **[mountloc]**
 special section. While mounting web-applications under [mountloc] additional 
-configuration files can be referred.
+configuration files can be referred. Example [mountloc] section,
 
-Example [mountloc] section,::
+.. code-block:: ini
 
   [mountloc]
   pluggdapps.com/issues = <appname>, <ini-file>
   tayra.pluggdapps.com/issues = <appname>, <ini-file>
   tayra.pluggdapps.com/source = <appname>, <ini-file>
 
-The referred configuration files are exclusive to the scope of the mounted
+The `lhs` side is called netpath which typically contains subdomain, hostname
+and scripth-path. The `rhs` side is a tuple of two elements. First is the
+name of a `IWebApp` plugin and second is path to application configuration
+file.
+
+Referred configuration files are exclusive to the scope of the mounted
 application, and shall not contain any special sections, except `[DEFAULT]`,
 unless otherwise explicitly mentioned. When a plugin is instantiated in the
 context of a web-application, configuration settings from application-ini-file
 will override settings from the master-ini-file.
-
-Finally the platform can be started like,::
-
-  pa = Webapps.boot( args.config )
-
-where ``args.config`` locates the master-ini file
 """
 
 from   configparser          import SafeConfigParser
-from   os.path               import dirname, isfile
+from   os.path               import dirname, isfile, abspath
 from   copy                  import deepcopy
+import curses
 
 from   pluggdapps.const      import SPECIAL_SECS, URLSEP
-from   pluggdapps.interfaces import IWebApp
+from   pluggdapps.interfaces import IWebApp, IConfigDB
 import pluggdapps.utils      as h
 
+SPECIAL_SECTIONS = ['DEFAULT', 'pluggdapps']
 
 def DEFAULT():
     """Global default settings that are applicable to all plugins and
@@ -105,11 +165,17 @@ def DEFAULT():
     section. Semantic meaning of [DEFAULT] section is same as described by
     ``configparser`` module from stdlib."""
     sett = h.ConfigDict()
-    sett.__doc__ = "Global configuration settings with system-wide scope."
+    sett.__doc__ = (
+        "Configuration settings under this section are global and applicable "
+        "every other sections."
+    )
     sett['debug']  = {
         'default'  : False,
         'types'    : (bool,),
-        'help'     : "Boot and run pluggdapps system in debug mode.",
+        'help'     : "Set this to True while developing with pluggdapps. This "
+                     "will enable useful logging and other features like "
+                     "module reloading. Can be modified only in the .ini "
+                     "file.",
         'webconfig': False,
     }
     return sett
@@ -124,34 +190,52 @@ def normalize_defaults( sett ):
 def pluggdapps_defaultsett():
     """Default settings for [pluggdapps] section in ini file."""
     sett = h.ConfigDict()
-    sett.__doc__ = "Pluggdapps platform settings."
-    sett['scheme'] = {
-        'default'  : 'http',
-        'types'    : (str,),
-        'help'     : "HTTP Scheme to use, either `http` or `https`."
+    sett.__doc__ = "Platform settings."
+    sett['configdb'] = {
+        'default'   : 'ConfigSqlite3DB',
+        'types'     : (str,),
+        'help'      : "Backend plugin to persist configurations done via "
+                      "webadmin application. Can be modified only in the .ini "
+                      "file.",
+        'webconfig' : False,
     }
     sett['host'] = {
-        'default'  : 'localhost',
-        'types'    : (str,),
-        'help'     : "Top level domain name of host, web server, which "
-                     "receives the request."
+        'default'   : 'localhost',
+        'types'     : (str,),
+        'help'      : "Top level domain name of host, web server, which "
+                      "receives the request. Can be modified only in the .ini "
+                      "file.",
+        'webconfig' : False,
     }
-    sett['port'] = {
-        'default' : 8080,
-        'types'   : (int,),
-        'help'    : "Port addres to bind the http server."
+    sett['logging.file'] = {
+        'default'   : '',
+        'types'     : (str,),
+        'help'      : "File name to log messages. Make sure to add `file` in "
+                      "`logging.output` parameter."
     }
     sett['logging.output'] = {
         'default' : 'console',
         'types'   : (str,),
         'help'    : "Comma separated value of names to log messages. Supported "
-                    "names are `console`, `file`."
+                    "names are `console`, `file`.",
+        'options' : [ 'console', 'file' ],
     }
-    sett['logging.file'] = {
-        'default' : '',
-        'types'   : (str,),
-        'help'    : "File name to log messages. Make sure to add `file` in "
-                    "`logging` parameter."
+    sett['port'] = {
+        'default'   : 8080,
+        'types'     : (int,),
+        'help'      : "Port addres to bind the http server. This configuration "
+                      "will be used when using pluggdapps' native HTTP "
+                      "server. Can be modified only in the .ini file.",
+        'webconfig' : False,
+    }
+    sett['scheme'] = {
+        'default'   : 'http',
+        'types'     : (str,),
+        'help'      : "HTTP Scheme to use, either `http` or `https`. This "
+                      "configuration will be used when using pluggdapps' "
+                      "native HTTP server. Can be modified only in the .ini "
+                      "file.",
+        'webconfig' : False,
     }
     return sett
 
@@ -163,6 +247,13 @@ def normalize_pluggdapps( sett ):
         sett['logging.output'] = h.parsecsv( sett['logging.output'] )
     return sett
 
+def mountloc_defaultsett():
+    sett = h.ConfigDict()
+    sett.__doc__ = "Mount application settings"
+    return sett
+
+def normalize_mountloc( sett ):
+    return sett
 
 class Pluggdapps( object ):
     """Platform class tying together pluggdapps platform, component
@@ -177,6 +268,9 @@ class Pluggdapps( object ):
     """Default settings for plugins, gathered from plugin-default settings,
     master configuration file and other backend stores, if any."""
 
+    configdb = None
+    """:class:`pluggdapps.interfaces.IConfigDB` plugin instance."""
+
     def __init__( self, erlport=None ):
         self.erlport = erlport # TODO: Document this once bolted with netscale
 
@@ -188,11 +282,42 @@ class Pluggdapps( object ):
         Return a new instance of this class object. This is the only way to
         create a platform instance.
         """
+        from pluggdapps import loadpackages, initialize
+
+        # Prebooting. We need pre-booting because package() entry point can
+        # have option of creating dynamic plugins when it is called. The
+        # package() entry point for each and every package will be called only
+        # when initialize() is called. Since boot() needs a platform
+        # context, we pre-boot the system for initialize then then actually
+        # boot the system.
+        loadpackages()
+
+        pa = Pluggdapps( *args, **kwargs )
+        pa.settings = pa._loadsettings( baseini )
+
+        # Configuration from backend store.
+        storetype = pa.settings['pluggdapps']['configdb']
+        configdb = pa.query_plugin( pa, IConfigDB, storetype )
+        configdb.dbinit()
+        dbsett = configdb.config()
+        if dbsett :
+            [ pa.settings[section].update(d) for section, d in dbsett.items() ]
+
+        pa.logsett = h.settingsfor( 'logging.', pa.settings['pluggdapps'] )
+        initialize( pa ) # Prebooting ends with initialize call.
+
+        # Actual booting
         pa = cls( *args, **kwargs )
         pa.inifile = baseini
         pa.settings = pa._loadsettings( baseini )
-        # Configure logging
+        pa.configdb = configdb
+
+        # Configuration from backend store
+        dbsett = pa.configdb.config()
+        if dbsett :
+            [ pa.settings[section].update(d) for section, d in dbsett.items() ]
         pa.logsett = h.settingsfor( 'logging.', pa.settings['pluggdapps'] )
+
         return pa
 
     def start( self ):
@@ -222,12 +347,37 @@ class Pluggdapps( object ):
         # Plugin settings
         plugin._settngx.update( kwargs.pop( 'settings', {} ))
 
-        plugin.query_plugins = hitch( plugin, plugin.__class__,
+        plugin.query_plugins = h.hitch_method( plugin, plugin.__class__,
                                       Pluggdapps.query_plugins, self )
-        plugin.query_plugin  = hitch( plugin, plugin.__class__,
+        plugin.query_plugin  = h.hitch_method( plugin, plugin.__class__,
                                       Pluggdapps.query_plugin, self )
         return args, kwargs
 
+    #---- Configuration APIs
+
+    def config( self, **kwargs ):
+        """Get or set configuration parameter for the platform. If no keyword
+        arguments are supplied, will return platform-settings.
+
+        Keyword arguments,
+
+        ``section``,
+            Section name to get or set config parameter. Optional.
+
+        ``name``,
+            Configuration name to get or set for ``section``. Optional.
+
+        ``value``,
+            If present, this method was invoked for setting configuration
+            ``name`` under ``section``. Optional.
+        """
+        section = kwargs.get( 'section', None )
+        name = kwargs.get( 'name', None )
+        value = kwargs.get( 'value', None )
+        if section and name and value :
+            self.settings[section][name] = value
+        return self.configdb.config( **kwargs )
+
     #---- Internal methods.
 
     def _loadsettings( self, inifile ):
@@ -246,11 +396,14 @@ class Pluggdapps( object ):
         and plugin sections."""
         from pluggdapps.plugin import pluginnames, plugin_info
 
+        if not baseini or (not isfile(baseini)) :
+            return deepcopy( defaultsett )
+
         # Initialize return dictionary.
         settings = {}
 
         # context for parsing ini files.
-        _vars = { 'here' : dirname(baseini) }
+        _vars = { 'here' : abspath( dirname( baseini )) }
 
         # Read master ini file.
         cp = SafeConfigParser()
@@ -265,6 +418,7 @@ class Pluggdapps( object ):
         s = deepcopy( defaultsett['pluggdapps'] )
         if cp.has_section('pluggdapps') :
             s.update( dict( cp.items( 'pluggdapps', vars=_vars )))
+            s.pop( 'here', None )   # TODO : how `here` gets populated ??
             settings['pluggdapps'] = normalize_pluggdapps( s )
 
         # Override plugin's package default settings with [DEFAULT] settings.
@@ -273,12 +427,12 @@ class Pluggdapps( object ):
             sett = h.mergedict( sett, settings['DEFAULT'] )
             if cp.has_section( pluginsec ) :
                 sett.update( dict( cp.items( pluginsec, vars=_vars )))
+                sett.pop( 'here', None )    # TODO : how `here` ??
             cls = plugin_info( h.sec2plugin( pluginsec ) )['cls']
             for b in reversed( cls.mro() ) :
                 if hasattr( b, 'normalize_settings' ) :
                     sett = b.normalize_settings( sett )
             settings[ pluginsec ] = sett
-
         return settings
 
     def _defaultsettings( self ):
@@ -308,8 +462,10 @@ class Pluggdapps( object ):
             name = info['name']
             bases = reversed( info['cls'].mro() )
             sett = deepcopy( default )
-            [ sett.update( dict( b.default_settings().items() )) 
-              for b in bases if hasattr(b, 'default_settings') ]
+            for b in bases :
+                if hasattr( b, 'default_settings' ) :
+                    sett.update( dict( b.default_settings().items() ))
+                    sett = b.normalize_settings( sett )
             defaultsett[ h.plugin2sec(name) ] = sett
 
         return defaultsett
@@ -362,7 +518,7 @@ class Pluggdapps( object ):
         If ``settings`` key-word argument is present, it will be used to
         override default plugin settings. Return a single Plugin instance.
         """
-        from pluggdapps.plugin import PluginMeta, pluginname
+        from pluggdapps.plugin import PluginMeta, pluginname, ISettings
         cls = PluginMeta._implementers[ interface ][ pluginname(name) ]
         return cls( pa, *args, **kwargs )
 
@@ -396,7 +552,7 @@ class Pluggdapps( object ):
         if 'file' in output and self.logsett['file'] :
             open( self.logsett['file'], 'a' ).write( formatstr )
         if 'console' in output :
-            print( colorize( formatstr, color='33' ))
+            print( h.colorize( formatstr, color='33' ))
 
     def logwarn( self, formatstr, values=[] ):
         """Use this method to log warning messages. The log messages will
@@ -410,7 +566,7 @@ class Pluggdapps( object ):
         if 'file' in output and self.logsett['file'] :
             open( self.logsett['file'], 'a' ).write( formatstr )
         if 'console' in output :
-            print( colorize( formatstr, color='32' ))
+            print( h.colorize( formatstr, color='32' ))
 
     def logerror( self, formatstr, values=[] ):
         """Use this method to log error messages. The log messages will
@@ -424,7 +580,7 @@ class Pluggdapps( object ):
         if 'file' in output and self.logsett['file'] :
             open( self.logsett['file'], 'a' ).write( formatstr )
         if 'console' in output :
-            print( colorize( formatstr, color='31' ))
+            print( h.colorize( formatstr, color='31' ))
 
 
 
@@ -436,6 +592,10 @@ class Webapps( Pluggdapps ):
     """Dictionay mapping of mount-key and :class:`pluggdapps.web.webapp.WebApp`
     instance, where mount-key is a tuple of appsec, netpath, application-ini"""
 
+    netpaths= {}
+    """Dictionay mapping of netpath, and :class:`pluggdapps.web.webapp.WebApp`
+    instance."""
+
     appurls = {}
     """A dictionary map of webapp's instkey and its base-url. The base url 
     consists of scheme, netloc, scriptname."""
@@ -443,9 +603,22 @@ class Webapps( Pluggdapps ):
     _app_resolve_cache = {}
     """A dictionary map of (netloc, script-path) to Web-application object."""
 
+    _monitoredfiles = []
+    """Attribute used in debug mode to collect and monitor files that will be
+    modified during developement."""
+
     def __init__( self, *args, **kwargs ):
         super().__init__( *args, **kwargs )
 
+    def findapp( self, appname=None, appsec=None ):
+        appsec = appsec or (h.plugin2sec( appname ) if appname else None)
+        if appsec :
+            for asec, netpath, config in self.webapps.keys() :
+                if appsec==asec : return self.webapps[(asec, netpath, config)]
+            else :
+                return None
+        return None
+
     #---- Overridable methods
 
     @classmethod
@@ -459,6 +632,9 @@ class Webapps( Pluggdapps ):
 
         appsettings = pa._mountapps()
 
+        netpaths = [ instkey[1] for instkey, appsett in appsettings.items() ]
+        pa.configdb.dbinit( netpaths=netpaths )
+
         # Mount webapp instances for subdomains and scripts.
         for instkey, appsett in appsettings.items() :
             appsec, netpath, config = instkey
@@ -466,11 +642,15 @@ class Webapps( Pluggdapps ):
             # Instantiate the IWebAPP plugin here for each `instkey`
             appname = h.sec2plugin( appsec )
             webapp = pa.query_plugin( pa, instkey, IWebApp, appname, appsett )
-            webapp.appsetting = appsett
+            webapp.appsettings = appsett
+            # Update with backend configuration.
+            [ webapp.appsettings[section].update( d )
+              for section, d in pa.configdb.config( netpath=netpath ).items() ]
+
             webapp.instkey, webapp.netpath = instkey, None
             pa.webapps[ instkey ] = webapp
+            pa.netpaths[ netpath ] = webapp
             pa.appurls[ instkey ] = webapp.baseurl = pa._make_appurl( instkey )
-
             
             pa._app_resolve_cache[ h.parse_netpath(netpath) ] = webapp
 
@@ -521,7 +701,7 @@ class Webapps( Pluggdapps ):
 
         elif webapp :                   # Not a IWebApp plugin
             plugin._settngx.update(
-                webapp.appsetting[ h.plugin2sec( pluginname( plugin )) ]
+                webapp.appsettings[ h.plugin2sec( pluginname( plugin )) ]
             )
             plugin.webapp = webapp
 
@@ -533,10 +713,10 @@ class Webapps( Pluggdapps ):
 
         plugin.pa = self
 
-        plugin.query_plugins = hitch( 
+        plugin.query_plugins = h.hitch_method( 
             plugin, plugin.__class__, Webapps.query_plugins,
             self, plugin.webapp )
-        plugin.query_plugin = hitch(
+        plugin.query_plugin = h.hitch_method(
             plugin, plugin.__class__, Webapps.query_plugin,
             self, plugin.webapp )
 
@@ -544,6 +724,44 @@ class Webapps( Pluggdapps ):
         plugin._settngx.update( kwargs.pop( 'settings', {} ))
         return args, kwargs
 
+    #---- Configuration APIs
+
+    def config( self, **kwargs ):
+        """Get or set configuration parameter for the platform. If no keyword
+        arguments are present, will return a dictionary of webapp-settings,
+        where each webapp is identified by netpath as key.
+
+        Keyword arguments,
+
+        ``netpath``,
+            Netpath, including hostname and script-path, on which
+            web-application is mounted. Optional.
+
+        ``section``,
+            Section name to get or set config parameter. Optional.
+
+        ``name``,
+            Configuration name to get or set for ``section``. Optional.
+
+        ``value``,
+            If present, this method was invoked for setting configuration
+            ``name`` under ``section``. Optional.
+        """
+        netpath = kwargs.get( 'netpath', None )
+        if netpath == None :
+            super().config( **kwargs )
+        else :
+            section = kwargs.get( 'section', None )
+            name = kwargs.get( 'name', None )
+            value = kwargs.get( 'value', None )
+            if netpath == 'platform' :
+                settings = self.settings
+            else :
+                settings = self.netpaths[ netpath ].appsettings
+            if section and name and value :
+                settings[section][name] = value
+            return self.configdb.config( **kwargs )
+
     #---- Internal methods
 
     def _mountapps( self ):
@@ -554,8 +772,7 @@ class Webapps( Pluggdapps ):
         settings = self.settings
 
         # context for parsing ini files.
-        _vars = { 'here' : dirname(self.inifile) }
-
+        _vars = { 'here' : abspath( dirname( self.inifile )) }
 
         # Fetch special section [mountloc]. And override them with [DEFAULT]
         # settings.
@@ -566,6 +783,7 @@ class Webapps( Pluggdapps ):
         else :
             mountloc = []
         settings['mountloc'] = dict( mountloc )
+        settings.pop( 'here', None ) # TODO : how `here` gets populated.
 
         # Parse mount configuration.
         appsecs = list( map( h.plugin2sec, webapps() ))
@@ -574,15 +792,17 @@ class Webapps( Pluggdapps ):
         for netpath, mounton in mountloc :
             if netpath in _skipopts : continue
 
-            parts = mounton.split(',', 1)
-            appname = parts.pop(0).strip()
-            configini = parts.pop(0).strip() if parts else None
+            parts = [ x.strip() for x in mounton.split(',', 1) ]
+            appname = parts.pop(0)
+            configini = parts.pop(0) if parts else None
             appsec = h.plugin2sec( appname )
 
             if appsec not in appsecs :
                 raise Exception("%r application not found." % appname )
+            if not configini :
+                raise Exception("configuration file %r not supplied"%configini)
             if not isfile( configini ) :
-                raise Exception("configuration file %r not present"%configini )
+                raise Exception("configuration file %r not valid"%configini)
 
             mountls.append( (appsec,netpath,configini) )
 
@@ -600,7 +820,7 @@ class Webapps( Pluggdapps ):
     def _loadinstance( self, appsett, instanceini ):
         """Load configuration settings for a web application's instance."""
         from pluggdapps.plugin import plugin_info
-        _vars = { 'here' : dirname(instanceini) }
+        _vars = { 'here' : abspath( dirname( instanceini )) }
         cp = SafeConfigParser()
         cp.read( instanceini )
 
@@ -613,6 +833,7 @@ class Webapps( Pluggdapps ):
         for sec in cp.sections() :
             if not sec.startswith( 'plugin:' ) : continue
             sett = dict( cp.items( sec, vars=_vars ))
+            sett.pop( 'here', None )    # TODO : how `here` gets populated ??
             appsett[sec].update( sett )
             cls = plugin_info( h.sec2plugin( sec ) )['cls']
             for b in reversed( cls.mro() ) :
@@ -627,14 +848,10 @@ class Webapps( Pluggdapps ):
 
         Return the base-url as byte-string.
         """
-        host = self.settings['pluggdapps']['host']
         port = self.settings['pluggdapps']['port']
         scheme = self.settings['pluggdapps']['scheme']
         appsec, netpath, config = instkey
         netloc, script = h.parse_netpath( netpath )
-        if not netloc.endswith( host ) :
-            raise Exception(
-                "Mount location's netloc does not match with host config." )
 
         # Prefix scheme
         appurl = scheme + '://' + netloc
@@ -684,8 +901,8 @@ class Webapps( Pluggdapps ):
 
     @staticmethod
     def query_plugin( pa, webapp, interface, name, *args, **kwargs ):
-        """Same as queryPlugins, but returns a single plugin instance as opposed
-        an entire list. `name` will be used to identify that plugin.
+        """Same as queryPlugins, but returns a single plugin instance as
+        opposed an entire list. `name` will be used to identify that plugin.
         Positional and keyword arguments will be used to instantiate the plugin
         object.
 
@@ -695,7 +912,7 @@ class Webapps( Pluggdapps ):
         ``webapp``,
             Web application object, plugin implementing
             :class:`pluggdapps.interfaces.IWebApp` interface. It is an
-            optional argument, which must be passed ``None` otherwise.
+            optional argument, which must be passed ``None`` otherwise.
 
         ``interface``,
             :class:`pluggdapps.plugin.Interface` class.
@@ -714,7 +931,6 @@ class Webapps( Pluggdapps ):
         return cls( pa, webapp, *args, **kwargs )
 
 
-
     #---- APIs related to hosting multiple-applications.
 
     def resolveapp( self, uri, hdrs ):
@@ -722,6 +938,7 @@ class Webapps( Pluggdapps ):
 
         ``uri``,
             byte-string of request URI.
+
         ``hdrs``,
             Dictionary of request headers.
 
@@ -729,8 +946,8 @@ class Webapps( Pluggdapps ):
             
         ``uriparts``,
             dictionary of URL parts 
-        ``mountedat``,
-            (type, mountname, webapp-instance)
+        ``webapp``,
+            :class:`IWebApp` plugin instance.
         """
         uriparts = h.parse_url( uri, host=hdrs.get('host', None) )
         for key, webapp in self._app_resolve_cache.items() :
@@ -740,38 +957,43 @@ class Webapps( Pluggdapps ):
                             else uriparts['host'] )
             if netloc == uri_netloc :
                 if script in ['/', ''] :
-                    return uriparts, webapp
+                    break
                 elif uriparts['path'].startswith( script ) :
                     uriparts['script'] = script
                     uriparts['path'] = uriparts['path'][len(script):]
-                    return uriparts, webapp
-        return uriparts, None
-
-    #---- ISettings interface methods
-
-    @classmethod
-    def default_settings( cls ):
-        return _default_settings
+                    break
+        else :
+            webapp = None
+        return uriparts, webapp
 
 
-def hitch( obj, cls, function, *args, **kwargs ) :
-    def fnhitched( self, *a, **kw ) :
-        kwargs.update( kw )
-        return function( *(args+a), **kwargs )
-    return fnhitched.__get__( obj, cls )
+class Consoleapps( Pluggdapps ):
+    """Provides a framework for console based applications using ncurses."""
+
+    stdscr = None
+    """Standard screen"""
+
+    def __init__( self, *args, **kwargs ):
+        super().__init__( *args, **kwargs )
+
+    #---- Overridable methods
+
+    def start( self ):
+        """Boot all loaded application. Web applicationss are loaded when the
+        system is booted via boot() call. Apps are booted only when an
+        explicit call is made to this method."""
+        super().start()
+        self.stdscr = curses.initscr()
+        self.stdscr.keypad(1)   # Handle multi-byte special characters
+        curses.noecho()         # Turn off echo
+        curses.cbreak()         # Non-buffered mode
+        curses.start_color()
+
+    def shutdown( self ):
+        """Reverse of start() method."""
+        curses.nocbreak()
+        curses.echo()
+        self.stdscr.keypad(0)
+        curses.endwin()
+        super().shutdown()
 
-def colorize( string, color, bold=False ):
-    """ Color values
-    Black       0;30     Dark Gray     1;30
-    Blue        0;34     Light Blue    1;34
-    Green       0;32     Light Green   1;32
-    Cyan        0;36     Light Cyan    1;36
-    Red         0;31     Light Red     1;31
-    Purple      0;35     Light Purple  1;35
-    Brown       0;33     Yellow        1;33
-    Light Gray  0;37     White         1;37
-    """
-    attr = []
-    attr.append( color )
-    attr.append('1') if bold else None
-    return '\x1b[%sm%s\x1b[0m' % (';'.join(attr), string)
