@@ -9,8 +9,8 @@
 # TODO :
 #   * Improve function asbool() implementation.
 
-import sys, os, fcntl, time, multiprocessing, random, io, traceback, hashlib
-import datetime as dt
+import sys, os, fcntl, multiprocessing, random, io, traceback, hashlib
+from   os.path  import isfile, join
 from   binascii import hexlify
 
 __all__ = [
@@ -18,9 +18,11 @@ __all__ = [
     'asbool', 'asint', 'asfloat', 'timedelta_to_seconds', 'set_close_exec', 
     'set_nonblocking', 'call_entrypoint', 'docstr', 'cpu_count', 
     'reseed_random', 'mergedict', 'multivalue_dict', 'takewhile', 
-    'dropwhile', 'print_exc', 'eval_import', 'string_import', 'str2module',
+    'dropwhile', 'flatten', 'print_exc', 'eval_import', 'string_import', 
+    'str2module', 'locatefile', 'hitch', 'hitch_method', 'colorize', 'strof',
+    'longest_prefix', 'dictsort', 'formated_filesize',
     # Classes
-    'Context',
+    'Context', 'Bunch',
 ]
 
 ver_int = int( str(sys.version_info[0]) + str(sys.version_info[1]) )
@@ -125,13 +127,8 @@ def call_entrypoint( distribution, group, name, *args, **kwargs ):
     using, ``distribution``, ``group`` and ``name``. ``args`` and ``kwargs``
     will be passed on to the entypoint callable.
     """
-    devmod = kwargs.pop( 'devmod', False )
-    try :
-        ep = distribution.get_entry_info( group, name )
-        return ep.load()( *args, **kwargs ) if ep else None
-    except :
-        if devmod : raise
-    return None
+    ep = distribution.get_entry_info( group, name )
+    return ep.load()( *args, **kwargs ) if ep else None
 
 
 def docstr( obj ):
@@ -182,6 +179,14 @@ def dropwhile( pred, lst ):
         i += 1
     return lst[i:]
 
+def flatten( lst ):
+    """Flatten nested list ``lst`` into a flat list."""
+    l = []
+    for e in lst :
+        if isinstance( e, list ) : l.extend( flatten( e ))
+        else : l.append( e )
+    return l
+
 def print_exc() :
     """Return a string representing exception info."""
     typ, val, tb = sys.exc_info()
@@ -210,8 +215,8 @@ def string_import( s ):
     """Import a module, object, or an attribute on an object.
 
     A string like ``foo.bar.baz`` can be a module ``foo.bar.baz`` or a
-    module ``foo.bar`` with an object ``baz`` in it, or a module
-    ``foo`` with an object ``bar`` with an attribute ``baz``.
+    module ``foo.bar`` with an object ``baz`` in it, or a module ``foo`` with
+    an object ``bar`` with an attribute ``baz``.
     """
     parts = s.split('.')
     module = str2module( parts[0] )
@@ -237,6 +242,104 @@ def str2module( s ):
         mod = getattr(mod, part)
     return mod
 
+def locatefile( phile, lookup_directories=[] ):
+    """Locate the absolute path in which file ``phile` is located."""
+
+    if phile.startswith('/') :
+        return phile if isfile(phile) else None
+
+    # First assume that file is relative to lookup_directories
+    files = list( filter( 
+               isfile, [ join(d, phile) for d in lookup_directories ] ))
+    if files : return files[0]
+
+    # Otherwise, assume that it is specified in asset specification notation
+    try    : return h.abspath_from_asset_spec( phile )
+    except : return None
+
+    raise Exception( 'Error locating TTL file %r' % phile )
+
+def hitch( function, *args, **kwargs ):
+    """Hitch a function with a different object and different set of
+    arguments."""
+    def fnhitched( *a, **kw ) :
+        kwargs.update( kw )
+        return function( *(args+a), **kwargs )
+    return fnhitched
+
+def hitch_method( obj, cls, function, *args, **kwargs ) :
+    def fnhitched( self, *a, **kw ) :
+        kwargs.update( kw )
+        return function( *(args+a), **kwargs )
+    return fnhitched.__get__( obj, cls )
+
+def colorize( string, color, bold=False ):
+    """ Color values
+    Black       0;30     Dark Gray     1;30
+    Blue        0;34     Light Blue    1;34
+    Green       0;32     Light Green   1;32
+    Cyan        0;36     Light Cyan    1;36
+    Red         0;31     Light Red     1;31
+    Purple      0;35     Light Purple  1;35
+    Brown       0;33     Yellow        1;33
+    Light Gray  0;37     White         1;37
+    """
+    attr = []
+    attr.append( color )
+    attr.append('1') if bold else None
+    return '\x1b[%sm%s\x1b[0m' % (';'.join(attr), string)
+
+def strof( o ):
+    """``o`` can be either a string or bytes or other object convertible to
+    string. In case of bytes, it will be decoded using 'utf-8'. In case of
+    None, None will be returned."""
+    if isinstance(o, str) : return o
+    elif isinstance(o, bytes) : return o.decode('utf-8')
+    elif o == None : return None
+    else : return str(o)
+
+def longest_prefix( prefix_l, s ) :
+    """Among the list of strings ``prefix_l``, find the longest prefix match
+    for string ``s``."""
+    result = None
+    for p in prefix_l :
+        if s.startswith( p ) :
+            result = p if (result==None) or (len(p) > len(result)) else result
+    return result
+
+def dictsort( d, case=False, by='key' ) :
+    """Sort a dict and yield (key, value) pairs."""
+    pos = { 'key' : 0, 'value' : 1 }[ by ]
+    fn = lambda x : x[pos] if case else x[pos].lower()
+    return sorted( d.items(), key=fn )
+
+def formated_filesize( size, binary=False ):
+    """Format file ``size`` to human-readable file size (i.e. 13 kB, 4.1 MB,
+    102 Bytes, etc).
+    """
+    # TODO : Should this be such a long function ??? 
+    value = float(size)
+    base = binary and 1024 or 1000
+    prefixes = [
+        (binary and 'KiB' or 'kB'),
+        (binary and 'MiB' or 'MB'),
+        (binary and 'GiB' or 'GB'),
+        (binary and 'TiB' or 'TB'),
+        (binary and 'PiB' or 'PB'),
+        (binary and 'EiB' or 'EB'),
+        (binary and 'ZiB' or 'ZB'),
+        (binary and 'YiB' or 'YB')
+    ]
+    if value == 1:
+        return '1 Byte'
+    elif value < base:
+        return '%d Bytes' % value
+    else:
+        for i, prefix in enumerate(prefixes):
+            unit = base ** (i + 2)
+            if value < unit :
+                return '%.1f %s' % ((base * value / unit), prefix)
+        return '%.1f %s' % ((base * value / unit), prefix)
 
 class ETag( dict ):
     """A dictionary like object to transparently manage context information.
@@ -257,19 +360,23 @@ class ETag( dict ):
     """
 
     def __init__( self, context, *args, **kwargs ):
+        """Override dict.__init__ to initalize internal data strucutres."""
         super().__init__( *args, **kwargs )
         self._c = context
         self._init()
 
     def __setitem__( self, key, value ):
+        """Override to populate the context object with key,value."""
         self._c[ key ] = value
         return super().__setitem__( key, value )
 
     def update( self, *args, **kwargs ):
+        """Override to populate the context object with *args and **kwargs."""
         self._c.update( *args, **kwargs )
         return super().update( *args, **kwargs )
 
     def setdefault( self, key, value=None ):
+        """Override to populate the context object with key, value."""
         self._c.setdefault( key, value )
         return super().setdefault( key, value )
 
@@ -282,17 +389,19 @@ class ETag( dict ):
 
     def hashout( self, prefix='', joinwith='', sep=';' ):
         """Return the hash digest so far."""
+        digest = '' # Initialize
         if self.values() or self._hashin :
             [ self._hasher.update(
                 v if isinstance(v, bytes) else str(v).encode('utf-8') 
               ) for v in self.values() ]
             self._hasher.update( self._hashin ) if self._hashin else None
             digest = prefix + self._hasher.hexdigest()
-        else :
-            digest = ''
         return sep.join( filter( None, [joinwith, digest ]))
 
     def clear( self ):
+        """Clear all key,value pairs so far populated on this dictionary
+        object and re-initialize the hash algorithm. Note that the same 
+        key, value pairs are still preserved in the context dictionary."""
         super().clear()
         self._init()
 
@@ -321,3 +430,19 @@ class Context( dict ):
     def __init__( self, *args, **kwargs ):
         super().__init__( *args, **kwargs )
         self.etag = ETag( self )
+
+
+class Bunch( object ):
+    """A generic container"""
+
+    def __init__( self, **attrs ):
+        [ setattr(self, name, value) for name, value in attrs.items() ]
+
+    def __repr__( self ):
+        name = '<%s ' % self.__class__.__name__
+        name += ' '.join(['%s=%r' % (name, str(value)[:30])
+                          for name, value in self.__dict__.items()
+                          if not name.startswith('_')])
+        return name + '>'
+
+
