@@ -14,10 +14,10 @@ import sys, datetime, errno, heapq, time, os, select, socket, re, \
 
 import ssl  # Python 2.6+
 
-import pluggdapps.utils             as h
-from   pluggdapps.plugin            import Plugin, implements
-from   pluggdapps.web.webinterfaces import IHTTPRequest
-from   pluggdapps.interfaces        import IHTTPServer, IHTTPConnection
+import pluggdapps.utils          as h
+from   pluggdapps.plugin         import Plugin, implements
+from   pluggdapps.web.interfaces import IHTTPRequest
+from   pluggdapps.interfaces     import IHTTPServer, IHTTPConnection
 
 
 # TODO :
@@ -25,109 +25,12 @@ from   pluggdapps.interfaces        import IHTTPServer, IHTTPConnection
 #     Request) status code to any HTTP/1.1 request message which lacks a Host
 #     header field.
 
-_ds1 = h.ConfigDict()
-_ds1.__doc__ = \
-    "Configuration settings for event poll based HTTP server."
-
-_ds1['scheme'] = {
-    'default'  : 'http',
-    'types'    : (str,),
-    'help'     : "HTTP Scheme to use, either `http` or `https`. If left "
-                 "empty `scheme` parameter from [pluggdapps] section will be "
-                 "used."
-}
-_ds1['host']  = {
-    'default' : '',
-    'types'   : (str,),
-    'help'    : "Address may be either an IP address or hostname.  If it's a "
-                "hostname, the server will listen on all IP addresses "
-                "associated with the name. Address may be an empty string "
-                "or None to listen on all available interfaces. Family may "
-                "be set to either ``socket.AF_INET`` or ``socket.AF_INET6`` "
-                "to restrict to ipv4 or ipv6 addresses, otherwise both will "
-                "be used if available. If left empty `host` parameter from "
-                "[pluggdapps] section will be used.",
-
-}
-_ds1['port']  = {
-    'default' : 0,
-    'types'   : (int,),
-    'help'    : "Port addres to bind the http server. If left empty `port` "
-                "paramter from [pluggdapps] section will be used."
-}
-_ds1['family'] = {
-    'default' : 'AF_INET',
-    'types'   : (str,),
-    'help'    : "Family may be set to either ``AF_INET`` or ``AF_INET6`` "
-                "to restrict to ipv4 or ipv6 addresses, otherwise both will "
-                "be used if available.",
-}
-_ds1['backlog']  = {
-    'default' : 128,
-    'types'   : (int,),
-    'help'    : "Back log of http request that can be queued at listening "
-                "port. This option is directly passed to socket.listen()."
-}
-_ds1['IHTTPConnection']  = {
-    'default' : 'httpconnection',
-    'types'   : (str,),
-    'help'    : "Plugin to handle client connections."
-}
-
-#---- SSL settings, for scheme `https`
-_ds1['ssl.certfile']  = {
-    'default' : '',
-    'types'   : (str,),
-    'help'    : "SSL Certificate file location.",
-}
-_ds1['ssl.keyfile']   = {
-    'default' : '',
-    'types'   : (str,),
-    'help'    : "SSL Key file location.",
-}
-_ds1['ssl.cert_reqs']  = {
-    'default' : ssl.CERT_REQUIRED,
-    'types'   : (int,),
-    'options' : [ ssl.CERT_NONE, ssl.CERT_OPTIONAL, ssl.CERT_REQUIRED ],
-    'help'    : "Whether a certificate is required from the other side of "
-                "the connection, and whether it will be validated if "
-                "provided. It must be one of the three values CERT_NONE "
-                "(certificates ignored), CERT_OPTIONAL (not required, but "
-                "validated if provided), or CERT_REQUIRED (required and "
-                "validated). If the value of this value is not CERT_NONE, "
-                "then the `ca_certs` parameter must point to a file of CA "
-                "certificates."
-}
-_ds1['ssl.ca_certs']   = {
-    'default' : None,
-    'types'   : (str,),
-    'help'    : "The ca_certs file contains a set of concatenated "
-                "certification authority. certificates, which are used to "
-                "validate certificates passed from the other end of the "
-                "connection."
-}
-#---- Setting for HTTPIOLoop
-_ds1['poll_threshold']     = {
-    'default' : 1000,
-    'types'   : (int,),
-    'help'    : "A warning limit for number of descriptors being polled by a "
-                "single poll instance. Will be used by HTTPIOLoop definition",
-}
-_ds1['poll_timeout']       = {
-    'default' : 3600.0,
-    'types'   : (float,),
-    'help'    : "In seconds. Poll instance will timeout after so many "
-                "seconds and perform callbacks (if any) and start a fresh "
-                "poll. Will be used by HTTPIOLoop definition",
-}
-
 def run_callback( server, callback, *args, **kwargs ):
     """Run a callback with `args`."""
     try:
         callback( *args, **kwargs )
     except Exception:
-        server.pa.logerror( "In callback %r" % callback )
-        if server['debug'] : raise
+        server.pa.logerror( h.print_exc() )
 
 class HTTPEPollServer( Plugin ):
     """A non-blocking, single-threaded HTTP Server plugin. `HTTPEPollServer`
@@ -137,11 +40,9 @@ class HTTPEPollServer( Plugin ):
     "keyfile".
 
     Server resolves application for HTTP requests and dispatches them to
-    corresponding :class:`IApplication` plugin. Finishing the request does
+    corresponding :class:`IWebApp` plugin. Finishing the request does
     not necessarily close the connection in the case of HTTP/1.1 keep-alive
-    requests. Server features,
-
-      * Basic connection handler parsing request startline, headers and body.
+    requests.
     """
 
     implements( IHTTPServer )
@@ -171,9 +72,10 @@ class HTTPEPollServer( Plugin ):
         except KeyboardInterrupt :
             self.stop()
         except :
-            self.pa.logdebug( h.print_exc() )
+            self.pa.logerror( h.print_exc() )
             self.stop()
         self.ioloop.close()
+        # Sanity check on unclosed connections
         if self.connections : 
             addrs = tuple( map( lambda c : c.address, self.connections ))
             self.pa.logwarn( "%r connections are still active" % (addrs,) )
@@ -191,15 +93,15 @@ class HTTPEPollServer( Plugin ):
         # Close listening sockets
         [ sock.close() for sock in self.sockets.values() ]
 
-    #---- Internal methods
-
     def close_connection( self, httpconn ):
-        """Close a client connection `httpconn` which is
-        :class:`IHTTPConnection` plugin."""
+        """:meth:`pluggdapps.interfaces.IHTTPServer.close_connection` 
+        interface method."""
         if httpconn in self.connections :
             self.pa.logdebug("Closing connection %r ..."%(httpconn.address,))
             self.connections.remove( httpconn )
 
+    #---- Internal methods
+
     def listen( self ):
         """Starts accepting connections on the given port. This method may be
         called more than once to listen on multiple ports.  `listen` takes
@@ -229,23 +131,16 @@ class HTTPEPollServer( Plugin ):
                 self.connections.append( httpconn )
 
         except ssl.SSLError as err:
-            self.pa.logerror( "Error(%s) in SSL connection %r" % (
-                              address, err.args[0] )
-                            )
+            self.pa.logerror( h.print_exc() )
             httpconn.close() if httpconn else None
-            if self['debug'] : raise
 
         except socket.error as err:
-            self.pa.logerror( "Socket Error(%s) in connection %r" % (
-                              address, err.args[0] )
-                            )
+            self.pa.logerror( h.print_exc() )
             httpconn.close() if httpconn else None
-            if self['debug'] : raise
 
         except Exception:
-            self.pa.logerror( "Error in accepting connection ... " )
+            self.pa.logerror( h.print_exc() )
             httpconn.close() if httpconn else None
-            if self['debug'] : raise
 
     def bind_sockets( self ):
         """Creates listening sockets (server) bound to the given port and 
@@ -336,6 +231,103 @@ class HTTPEPollServer( Plugin ):
         return sett
 
 
+_ds1 = h.ConfigDict()
+_ds1.__doc__ = HTTPEPollServer.__doc__
+
+_ds1['IHTTPConnection']  = {
+    'default' : 'httpconnection',
+    'types'   : (str,),
+    'help'    : "Plugin to handle client connections."
+}
+_ds1['backlog']  = {
+    'default' : 128,
+    'types'   : (int,),
+    'help'    : "Back log of http request that can be queued at listening "
+                "port. This option is directly passed to socket.listen()."
+}
+_ds1['family'] = {
+    'default' : 'AF_INET',
+    'types'   : (str,),
+    'help'    : "Family may be set to either ``AF_INET`` or ``AF_INET6`` "
+                "to restrict to ipv4 or ipv6 addresses, otherwise both will "
+                "be used if available.",
+    'options' : ['AF_NET', 'AF_INET6'],
+}
+_ds1['host']  = {
+    'default' : '',
+    'types'   : (str,),
+    'help'    : "Address may be either an IP address or hostname.  If it's a "
+                "hostname, the server will listen on all IP addresses "
+                "associated with the name. Address may be an empty string "
+                "or None to listen on all available interfaces. Family may "
+                "be set to either ``socket.AF_INET`` or ``socket.AF_INET6`` "
+                "to restrict to ipv4 or ipv6 addresses, otherwise both will "
+                "be used if available. If left empty `host` parameter from "
+                "[pluggdapps] section will be used.",
+
+}
+_ds1['port']  = {
+    'default' : 0,
+    'types'   : (int,),
+    'help'    : "Port addres to bind the http server. If left empty `port` "
+                "paramter from [pluggdapps] section will be used."
+}
+_ds1['scheme'] = {
+    'default'  : 'http',
+    'types'    : (str,),
+    'help'     : "HTTP Scheme to use, either `http` or `https`. If left "
+                 "empty `scheme` parameter from [pluggdapps] section will be "
+                 "used."
+}
+#---- Setting for HTTPIOLoop
+_ds1['poll_threshold']     = {
+    'default' : 1000,
+    'types'   : (int,),
+    'help'    : "A warning limit for number of descriptors being polled by a "
+                "single poll instance. Will be used by HTTPIOLoop plugin.",
+}
+_ds1['poll_timeout']       = {
+    'default' : 3600.0,
+    'types'   : (float,),
+    'help'    : "Poll instance will timeout after the specified number of "
+                "seconds and perform callbacks (if any) and start a fresh "
+                "poll. Will be used by HTTPIOLoop definition",
+}
+#---- SSL settings, for scheme `https`
+_ds1['ssl.certfile']  = {
+    'default' : '',
+    'types'   : (str,),
+    'help'    : "SSL Certificate file location. SSL options can be set only "
+                "in the .ini file.",
+}
+_ds1['ssl.keyfile']   = {
+    'default' : '',
+    'types'   : (str,),
+    'help'    : "SSL Key file location. SSL options can be set only in the "
+                ".ini file."
+}
+_ds1['ssl.cert_reqs']  = {
+    'default' : ssl.CERT_REQUIRED,
+    'types'   : (int,),
+    'options' : [ ssl.CERT_NONE, ssl.CERT_OPTIONAL, ssl.CERT_REQUIRED ],
+    'help'    : "Whether a certificate is required from the other side of "
+                "the connection, and whether it will be validated if "
+                "provided. It must be one of the three values CERT_NONE "
+                "(certificates ignored), CERT_OPTIONAL (not required, but "
+                "validated if provided), or CERT_REQUIRED (required and "
+                "validated). If the value of this value is not CERT_NONE, "
+                "then the `ca_certs` parameter must point to a file of CA "
+                "certificates. SSL options can be set only in the .ini file."
+}
+_ds1['ssl.ca_certs']   = {
+    'default' : None,
+    'types'   : (str,),
+    'help'    : "The ca_certs file contains a set of concatenated "
+                "certification authority. certificates, which are used to "
+                "validate certificates passed from the other end of the "
+                "connection. SSL options can be set only in the .ini file."
+}
+
 def add_accept_handler( server, sock, callback, ioloop ):
     """Adds an ``IOLoop`` event handler to accept new connections on 
     ``sock``. When a connection is accepted, ``callback(connection, address)``
@@ -352,8 +344,7 @@ def add_accept_handler( server, sock, callback, ioloop ):
             except socket.error as e:
                 if e.args[0] in (errno.EWOULDBLOCK, errno.EAGAIN):
                     return
-                server.pa.logdebug( "Unable to accept connection." )
-                if self['debug'] : raise
+                server.pa.logerror( h.print_exc() )
 
             server.pa.logdebug( "Accepting new connection from %r"%(address,) )
             callback( connection, address )
@@ -413,7 +404,6 @@ class IOLoop( object ):
         self.poll_threshold = server['poll_threshold']
         self.poll_timeout = server['poll_timeout']
         self.server = server
-        self.debug = server['debug']
 
         self._evpoll = select.epoll()
         self._waker = Waker()
@@ -428,9 +418,7 @@ class IOLoop( object ):
         self._running = False
         self._stopped = False
 
-        if self.debug :
-            server.pa.logdebug( "Adding poll-loop waker ..." )
-
+        server.pa.logdebug( "Adding poll-loop waker ..." )
         self.add_handler( self._waker.fileno(), 
                           lambda fd, events: self._waker.consume(), # handler
                           self.READ )
@@ -449,27 +437,22 @@ class IOLoop( object ):
             self.server.pa.logwarn(
                 "Polled descriptors exceeded threshold" % self.poll_threshold
             )
-        if self.debug :
-            self.server.pa.logdebug(
-                "Add descriptor to epoll : %s" % list( self._handlers.keys() )
-            )
+        self.server.pa.logdebug( "Add descriptor to epoll : %s" % fd )
 
     def update_handler( self, fd, events ):
         """Changes the events we listen for fd."""
         self._evpoll.modify( fd, events | self.ERROR )
-        if self.debug :
-            self.server.pa.logdebug(
-                "Updating descriptor : (%s, %s)" % (fd, events)
-            )
+        self.server.pa.logdebug("Updating descriptor : %s, %s" % (fd, events))
 
     def remove_handler( self, fd ):
         """Stop listening for events on fd."""
-        if self._handlers.pop(fd, None) or self._events.pop(fd, None) :
-            self.server.pa.logdebug("Remove descriptor from epoll : %s:" % fd)
-            try:
-                self._evpoll.unregister(fd)
-            except (OSError, IOError):
-                self.server.pa.logwarn( "Error deleting fd from epoll" )
+        self.server.pa.logdebug( "Remove descriptor from epoll : %s:" % fd )
+        self._handlers.pop(fd, None)
+        self._events.pop(fd, None)
+        try:
+            self._evpoll.unregister(fd)
+        except (OSError, IOError):
+            self.server.pa.logwarn( "Error deleting fd from epoll" )
 
     #---- Manage timeout handlers on this epoll using heap queue.
 
@@ -535,9 +518,7 @@ class IOLoop( object ):
             self._callbacks = []
             for callback in callbacks :
                 try    : callback()
-                except :
-                    self.server.pa.logerror( "In callback %r" % callback )
-                    if self.debug : raise
+                except : self.server.pa.logerror( h.print_exc() )
 
             # Handle timeouts
             if self._timeouts :
@@ -549,9 +530,7 @@ class IOLoop( object ):
                     elif self._timeouts[0].deadline <= now : # Handle timeout
                         timeout = heapq.heappop( self._timeouts )
                         try    : timeout.callback()
-                        except :
-                            self.server.pa.logerror("In timeout %r" % timeout)
-                            if self.debug : raise
+                        except : self.server.pa.logerror( h.print_exc() )
 
                     else : # Adjust poll-timeout
                         seconds = self._timeouts[0].deadline - now
@@ -587,10 +566,8 @@ class IOLoop( object ):
             while self._events :
                 fd, events = self._events.popitem()
                 callback = self._handlers.get( fd, None )
-                try : callback( fd, events ) if callback else None
-                except :
-                    self.server.pa.logerror("In socket callback %r"%callback)
-                    if self.server['debug'] : raise
+                try    : callback( fd, events ) if callback else None
+                except : self.server.pa.logerror( h.print_exc() )
 
         # reset the stopped flag so another start/stop pair can be issued
         self._stopped = False
@@ -607,13 +584,15 @@ class IOLoop( object ):
         """
         # Close listening sockets and remove them from IOLoop
         self.server.pa.logdebug( "Stopping poll loop ..." )
-        self.remove_handler( self._waker.fileno() )
 
-        fds = list( self._handlers.keys() )
-        [ self.remove_handler( fd ) for fd in fds[:] ]
+        for fd in list(self._handlers.keys()) :
+            # Don't remove the waker, should  be done in close() method.
+            if fd is self._waker.fileno() : continue
+            self.remove_handler( fd ) 
 
         self._running = False
         self._stopped = True
+
         self._waker.wake()  # Wake the ioloop
 
     def close( self ):
@@ -625,57 +604,32 @@ class IOLoop( object ):
         call `IOLoop.close()`. Therefore the call to `close` will usually 
         appear just after the call to `start` rather than near the call to 
         `stop`. """
+        # Waker is only removed here. Don't change this sequence.
+        self.remove_handler( self._waker.fileno() )
+
         self._waker.close()
         self._evpoll.close()
-
         # Remove all the references to other objects, so that it will be
         # garbage collected.
         self._waker = self._evpoll = self.server = None
         self._callbacks = []
         self._timeouts = []
         if self._handlers :
-            raise Exception(
+            self.server.pa.logerror( 
                     "Handlers are still subscribed: %r" % self._handlers )
         if self._events :
-            raise Exception(
+            self.server.pa.logerror(
                     "Events are pending to be handled: %r" % self._events )
 
 
-
-_ds2 = h.ConfigDict()
-_ds2.__doc__ = \
-    "Configuration settings for event poll based HTTP server."
-
-_ds2['no_keep_alive']  = {
-    'default' : False,
-    'types'   : (bool,),
-    'help'    : "HTTP /1.1, whether to close the connection after every "
-                "request.",
-}
-_ds2['connection_timeout']  = {
-    'default' : 60*60*1,    # 1 hours
-    'types'   : (int,),
-    'help'    : "Timeout after which an idle connection is gracefully closed."
-}
-_ds2['max_buffer_size'] = {
-    'default' : 104857600,  # 100MB
-    'types'   : (int,),
-    'help'    : "Maximum size of read / write buffer."
-}
-_ds2['read_chunk_size'] = {
-    'default' : 4096,
-    'types'   : (int,),
-    'help'    : "Chunk of data to read at a time."
-}
-
-
 class HTTPConnection( Plugin ):
-    """Handles a connection to an HTTP client, executing HTTP requests. We 
-    parse HTTP headers and bodies, and execute the request callback until the
-    HTTP conection is closed.
+    """:class:`IHTTPConnection` plugin to handle http connections. Every
+    client connection corresponds to an instance of this plugin, handle
+    request, parse headers and bodies, and execute the request callback and 
+    writes the response back.
 
-      * Accepts only HTTP/1.1 request. If otherwise, reponds with bad-request
-        (400) and closes the connection.
+    Accepts only HTTP/1.1 request. If otherwise, reponds with bad-request
+    (400) and closes the connection.
     """
 
     implements( IHTTPConnection )
@@ -737,9 +691,9 @@ class HTTPConnection( Plugin ):
         self.chunk = None
 
         # Set up a socket from accepted connection (conn, addr).
-        sslopts = h.settingsfor( 'ssl.', server )
         scheme = server['scheme'] or self.pa.settings['pluggdapps']['scheme']
         if scheme == 'https' :
+            sslopts = h.settingsfor( 'ssl.', server )
             self.conn = ssl.wrap_socket( conn, server_side=True,
                                          do_handshake_on_connect=False,
                                          **sslopts )
@@ -781,8 +735,7 @@ class HTTPConnection( Plugin ):
         # Fresh request, resolve application.
         uriparts, webapp = self.pa.resolveapp( uri, headers )
         if webapp == None :
-            self.pa.logerror(
-                "Unable to resolve request for apps. (%s)" % uri )
+            self.pa.logerror("Unable to resolve request for apps. (%s)" % uri)
             self.write_error( self.NOT_FOUND )
             return
 
@@ -791,9 +744,9 @@ class HTTPConnection( Plugin ):
             # of a webapp, use `webapp` plugin to query for IHTTPRequest.
             request = webapp.query_plugin(
                             IHTTPRequest, webapp['IHTTPRequest'],
-                            self, method, uri, version, headers )
+                            self, method, uri, uriparts, version, headers )
         except :
-            h.print_exc()
+            self.pa.logerror( h.print_exc() )
             self.write_error( self.INTERNAL_ERROR )
             return
 
@@ -814,8 +767,7 @@ class HTTPConnection( Plugin ):
         """
         response = getattr( self.request, 'response', None )
         if self.request == None :
-            self.pa.logerror( "Request is not yet received." )
-            return
+            raise Exception( "Request is not yet received." )
 
         if self.stream and self.stream.closed() :
             self.pa.logwarn("Cannot write to closed stream %r"%(self.address,))
@@ -837,7 +789,7 @@ class HTTPConnection( Plugin ):
         if self.stream and self.stream.closed() :
             self.pa.logwarn("Cannot write to closed stream %r"%(self.address,))
             return
-        self.stream.write( rawdata, self.tryclose )
+        self.stream.write( rawdata, self.close )
         return
 
     def supports_http_1_1( self ):
@@ -852,7 +804,7 @@ class HTTPConnection( Plugin ):
         guidelines to close the connection.""" 
         if disconnect == False and self.request :
             meth, hdrs = self.request.method, self.request.headers
-            conn_val = h.connection( hdrs.get( "connection", None ))
+            conn_val = h.parse_connection( hdrs.get( "connection", b'' ))
             disconnect = conn_val == [ b'close' ]
 
         if disconnect == True :
@@ -893,7 +845,7 @@ class HTTPConnection( Plugin ):
     def on_request_headers( self, data ):
         """A request has started. Parse `data` for startline and headers."""
         if self.request != None :
-            h.print_exc()
+            self.pa.logerror( h.print_exc() )
             self.write_error( self.INTERNAL_ERROR )
             return
 
@@ -921,9 +873,9 @@ class HTTPConnection( Plugin ):
             # The presence of a message-body in a request is signaled by the
             # inclusion of a Content-Length or Transfer-Encoding header field
             # in the request's message-headers.
-            clen = h.content_length( hdrs.get( "content_length", None ))
-            transenc = h.transfer_encoding( 
-                            hdrs.get( 'transfer_encoding', None ))
+            clen = h.parse_content_length( hdrs.get( "content_length", None ))
+            transenc = h.parse_transfer_encoding( 
+                            hdrs.get( 'transfer_encoding', b'' ))
 
             if transenc and transenc[0][0] == b'chunked' :
                 hdrs.pop( "content_length", None )
@@ -935,7 +887,7 @@ class HTTPConnection( Plugin ):
                     self.write_error( self.ENTITY_LARGE )
                     self.stream.read_bytes( clen, self.on_skip_request )
 
-                elif expect == "100-continue" :
+                elif expect == b"100-continue" :
                     self.stream.write( b"HTTP/1.1 100 (Continue)\r\n\r\n" )
                     self.stream.read_bytes( clen, self.on_request_body )
 
@@ -949,7 +901,7 @@ class HTTPConnection( Plugin ):
                             b"\r\n\r\n", self.on_request_headers )
 
         except :
-            h.print_exc()
+            self.pa.logerror( h.print_exc() )
             self.write_error( self.BAD_REQUEST )
         return
 
@@ -978,7 +930,7 @@ class HTTPConnection( Plugin ):
 
         if chunk_size == 0 :    # last_chunk
             self.chunk = (chunk_size, chunk_ext, None)
-            if h.trailer( hdrs.get( 'trailer', None )) :# Trailer present
+            if hdrs.get( 'trailer', None ) :    # Trailer present
                 self.stream.read_until( b"\r\n\r\n", self.on_request_trailer )
             else :
                 self.stream.read_until( b"\r\n", self.on_request_chunks_done )
@@ -1050,6 +1002,34 @@ class HTTPConnection( Plugin ):
         return sett
 
 
+_ds2 = h.ConfigDict()
+_ds2.__doc__ = HTTPConnection.__doc__
+
+_ds2['connection_timeout']  = {
+    'default' : 60*60*1,    # 1 hours
+    'types'   : (int,),
+    'help'    : "Timeout in seconds after which an idle connection is "
+                "gracefully closed."
+}
+_ds2['max_buffer_size'] = {
+    'default' : 104857600,  # 100MB
+    'types'   : (int,),
+    'help'    : "Maximum size of read / write buffer in bytes."
+}
+_ds2['no_keep_alive']  = {
+    'default' : False,
+    'types'   : (bool,),
+    'help'    : "HTTP /1.1, whether to close the connection after every "
+                "request.",
+}
+_ds2['read_chunk_size'] = {
+    'default' : 4096,
+    'types'   : (int,),
+    'help'    : "Chunk of data, size in bytes, to read at a time."
+}
+
+
+
 class Timeout( object ):
     """Heapable timeout, a UNIX timestamp and a callback"""
 
@@ -1091,8 +1071,7 @@ class Waker( object ):
         return self.reader.fileno()
 
     def wake(self):
-        try           : self.writer.write( b'x' )
-        except IOError: pass
+        self.writer.write( b'wake up epoll' )
 
     def consume(self):
         try :
@@ -1302,8 +1281,12 @@ class IOStream( object ):
         # If the socket is not closed, then try reading from the socket.
         try :
             while self.read_to_buffer() : pass
-        except :
-            self.server.pa.logdebug( h.print_exc() )
+        except Exception as e :
+            if e.args[0] == 'Closed' :
+                self.server.pa.logwarn(
+                    "May be remote end %r closed" % (self.address,) )
+            else :
+                self.server.pa.logerror( h.print_exc() )
             self.close()
 
         # And see if we've already got the data from this read
@@ -1423,13 +1406,13 @@ class IOStream( object ):
                 state |= self.ioloop.READ
 
             if state != self._state:
-                if self._state is not None :
+                if self._state is None :
                     raise Exception( 
                             "shouldn't happen: on_epoll_event without _state" )
                 self._state = state
                 self.ioloop.update_handler( self.conn.fileno(), self._state )
         except Exception :
-            self.server.pa.logdebug( h.print_exc() )
+            self.server.pa.logerror( h.print_exc() )
             self.close()
 
     def handle_read( self ):
@@ -1442,8 +1425,12 @@ class IOStream( object ):
                 # out if it's there is to try to read it.
                 if self.read_to_buffer() == 0:
                     break
-        except :
-            self.server.pa.logdebug( h.print_exc() )
+        except Exception as e :
+            if e.args[0] == 'Closed' :
+                self.server.pa.logwarn(
+                    "May be remote end %r closed" % (self.address,) )
+            else :
+                self.server.pa.logerror( h.print_exc() )
             self.close()
         self.try_read_buffer()
 
@@ -1459,7 +1446,7 @@ class IOStream( object ):
             raise
         # May be the remote end closed
         if not chunk :
-            raise Exception( "May be remote end %r closed" % (self.address,) )
+            raise Exception("Closed")
         return chunk
 
     def read_to_buffer(self):
@@ -1469,11 +1456,8 @@ class IOStream( object ):
         to read (i.e. the read returns EWOULDBLOCK or equivalent).  On
         error closes the socket and raises an exception.
         """
-        try:
-            self.check_closed()
-            chunk = self.read_from_socket()
-        except :
-            raise
+        self.check_closed()
+        chunk = self.read_from_socket()
 
         chunklen = 0
         if chunk is not None :
@@ -1514,7 +1498,7 @@ class IOStream( object ):
                     self._write_buffer_frozen = True
                     break
                 else:
-                    self.server.pa.logdebug( h.print_exc() )
+                    self.server.pa.logerror( h.print_exc() )
                     self.close()
                     return
 
